@@ -5,7 +5,6 @@
 
 #include <linux/clk.h>
 #include <linux/delay.h>
-#include <linux/dma-mapping.h>
 #include <linux/host1x.h>
 #include <linux/iommu.h>
 #include <linux/module.h>
@@ -233,8 +232,10 @@ static int vic_load_firmware(struct vic *vic)
 
 	if (!client->group) {
 		virt = dma_alloc_coherent(vic->dev, size, &iova, GFP_KERNEL);
-		if (!virt)
-			return -ENOMEM;
+
+		err = dma_mapping_error(vic->dev, iova);
+		if (err < 0)
+			return err;
 	} else {
 		virt = tegra_drm_alloc(tegra, size, &iova);
 	}
@@ -275,7 +276,7 @@ cleanup:
 }
 
 
-static int __maybe_unused vic_runtime_resume(struct device *dev)
+static int vic_runtime_resume(struct device *dev)
 {
 	struct vic *vic = dev_get_drvdata(dev);
 	int err;
@@ -309,7 +310,7 @@ disable:
 	return err;
 }
 
-static int __maybe_unused vic_runtime_suspend(struct device *dev)
+static int vic_runtime_suspend(struct device *dev)
 {
 	struct vic *vic = dev_get_drvdata(dev);
 	int err;
