@@ -2505,13 +2505,7 @@ int qlcnic_83xx_init(struct qlcnic_adapter *adapter, int pci_using_dac)
 		goto disable_mbx_intr;
 
 	qlcnic_83xx_clear_function_resources(adapter);
-
-	err = qlcnic_dcb_enable(adapter->dcb);
-	if (err) {
-		qlcnic_dcb_free(adapter->dcb);
-		goto disable_mbx_intr;
-	}
-
+	qlcnic_dcb_enable(adapter->dcb);
 	qlcnic_83xx_initialize_nic(adapter, 1);
 	qlcnic_dcb_get_info(adapter->dcb);
 
